/*
 * say_tw.c
 */

/*! \brief  say_number_full_tw: Taiwanese syntax */
static int say_number_full_tw(const say_args_t *a, int num, const char *options)
{
	int res = 0;
	int playh = 0;
	char fn[256] = "";
	if (!num)
		return say_digits_full(a, 0);

	while(!res && (num || playh)) {
		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus");
			num = negate(num);
		} else if (playh) {
			snprintf(fn, sizeof(fn), "digits/hundred");
			playh = 0;
		} else	if (num < 10) {
			snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else	if (num < 100) {
			snprintf(fn, sizeof(fn), "digits/%d", (num /10) * 10);
			num -= ((num / 10) * 10);
		} else if (num < ONE_THOUSAND){
			snprintf(fn, sizeof(fn), "digits/%d", (num/100));
			playh++;
			num -= ((num / 100) * 100);
		} else if (num < ONE_MILLION) { /* 1,000,000 */
			res = say_number_full_tw(a, num / ONE_THOUSAND, options);
			if (res)
				return res;
			num = num % ONE_THOUSAND;
			snprintf(fn, sizeof(fn), "digits/thousand");
		} else if (num < ONE_BILLION) {	/* 1,000,000,000 */
			res = say_number_full_tw(a, num / ONE_MILLION, options);
			if (res)
				return res;
			num = num % ONE_MILLION;
			snprintf(fn, sizeof(fn), "digits/million");
		} else {
			ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
			res = -1;
		}
		if (!res)
			res = s_streamwait3(a, fn);
	}
	return res;
}

/* Taiwanese syntax */
static int say_date_with_format_tw(const say_args_t *a, const struct tm *tm1,
	const char *format, time_t delta)
{
	struct tm tm = *tm1;
	int res=0, offset;
	char c;
	say_date_with_format_f *f = say_date_with_format_tw; /* shorthand for recursive call */

	for (offset=0 ; res == 0 && (c = format[offset]) != '\0' ; offset++) {
		ast_log(LOG_DEBUG, "Parsing %c (offset %d) in %s\n", format[offset], offset, format);
		switch (c) {
		case 'm':
			/* First - Twelfth */
			res = s_streamwait(a, "digits/h-%d", tm.tm_mon +1);
			break;
		case 'd':
		case 'e':
			/* First - Thirtyfirst */
			if (!(tm.tm_mday % 10) || (tm.tm_mday < 10)) {
				res = s_streamwait(a, "digits/h-%d", tm.tm_mday);
			} else {
				res = s_streamwait(a, "digits/h-%dh", tm.tm_mday - (tm.tm_mday % 10));
				if(!res) {
					res = s_streamwait(a, "digits/h-%d", tm.tm_mday % 10);
				}
			}
			break;
		case 'Y':
			/* Year */
			if (tm.tm_year > 99) {
				res = s_streamwait3(a,  "digits/2");
				if (!res)
					res = s_streamwait3(a,  "digits/thousand");
				if (tm.tm_year > 100) {
					if (!res) {
						res = s_streamwait(a, "digits/%d", (tm.tm_year - 100) / 10);
						if (!res)
							res = s_streamwait(a, "digits/%d", (tm.tm_year - 100) % 10);
					}
				}
			} else if (tm.tm_year < 1) {
				/* I'm not going to handle 1900 and prior */
				/* We'll just be silent on the year, instead of bombing out. */
				break;
			} else {
				res = s_streamwait3(a,  "digits/1");
				if (res)
					break;
				res = s_streamwait3(a,  "digits/9");
				if (res)
					break;
				if (tm.tm_year <= 9) {
					/* 1901 - 1909 */
					res = s_streamwait3(a,  "digits/0");
					if (!res)
						res = s_streamwait(a, "digits/%d", tm.tm_year);
				} else {
					/* 1910 - 1999 */
					res = s_streamwait(a, "digits/%d", tm.tm_year / 10);
					if (!res)
						res = s_streamwait(a, "digits/%d", tm.tm_year % 10);
				}
			}
			if (!res)
				res = s_streamwait3(a,  "digits/year");
			break;
		case 'I':
		case 'l':
			/* 12-Hour */
			res = s_streamwait(a, "digits/%d", hr_to_12(tm.tm_hour));
			if (!res)
				res = s_streamwait3(a,  "digits/oclock");
			break;
		case 'H':
		case 'k':
			/* 24-Hour */
			if (!(tm.tm_hour % 10) || tm.tm_hour < 10) {
				if (tm.tm_hour < 10)
					res = s_streamwait3(a, "digits/0");
				res = s_streamwait(a, "digits/%d", tm.tm_hour);
			} else {
				res = s_streamwait(a, "digits/%d", tm.tm_hour - (tm.tm_hour % 10));
				if (!res)
					res = s_streamwait(a, "digits/%d", tm.tm_hour % 10);
			}
			if (!res)
				res = s_streamwait3(a,  "digits/oclock");
			break;
		case 'M':
			/* Minute */
			if (!(tm.tm_min % 10) || tm.tm_min < 10) {
				if (tm.tm_min < 10)
					res = s_streamwait3(a, "digits/0");
				res = s_streamwait(a, "digits/%d", tm.tm_min);
			} else {
				res = s_streamwait(a, "digits/%d", tm.tm_min - (tm.tm_min % 10));
				if (!res)
					res = s_streamwait(a, "digits/%d", tm.tm_min % 10);
			}
			if (!res)
				res = s_streamwait3(a,  "digits/minute");
			break;
		case 'S':
			/* Seconds */
			if (!(tm.tm_sec % 10) || tm.tm_sec < 10) {
				if (tm.tm_sec < 10)
					res = s_streamwait3(a, "digits/0");
				res = s_streamwait(a, "digits/%d", tm.tm_sec);
			} else {
				res = s_streamwait(a, "digits/%d", tm.tm_sec - (tm.tm_sec % 10));
				if (!res)
					res = s_streamwait(a, "digits/%d", tm.tm_sec % 10);
			}
			if (!res)
				res = s_streamwait3(a,  "digits/second");
			break;
		case 'x':
			res = f(a, &tm, "YBdA", delta);
			break;

		default:
			res = say_date_common(a, &tm, format, delta, &offset, f);
		}
	}
	return res;
}

static struct _lang_sw map_tw = {
	.language = "tw",
	.say_number_full = say_number_full_tw,
	.say_date_with_format = say_date_with_format_tw,
	.time_fmt = "PIM",
	.datetime_fmt = "YBdAPIM",
};

